#!/usr/local/bin/perl -w
#File name = dbexplorer.pl
#Version 0.1
#RJJ 000815
#updated RJJ 010314
#
#Aim: 	Digest each protein in a database
#     	Grab protein information
#		Accession No.
#		MS-Digest No.
#   	No of AA
#		PI
#		MW
#		No. of peptides
#		No of AA containing peptides
#		MW of AA containing peptides
#
#Output in to a big tab delimited text file. 
#
#Update
#Change to any single amino acid

#Update 3/7/02
#remove dependence on Protein Prospector
#Read in and process database from disk
#digest proteins directly
#do all AA in one run

#Added support for calculating CAF peptides

#added GUI

#updated GUI

#Removed support for CAF peptides

#addedstats generating sub
#cleaned up GUI

#added retention time calculation code

#Added AM and AMT calculation code


use strict; #enforce variable declarations and quoting
use XML::Simple;
use Getopt::Std;
use Tk;
use File::Basename;
use Tk::FileDialog;
require Tk::ActivityBar;
require Tk::DialogBox;
use Tk::Carp qw(warningsToDialog fatalsToDialog);


#Command line inputs
my (%opts,
	$commandlineSwitch,
	$database_name,
	$enzymeName,
	$doBinningBit,
	$DBFiletype
	);
getopts("hcbd:e:f:", \%opts);
usage() if $opts{h};

#are we running on the command line
if ($opts{c}){
	$commandlineSwitch =1;
	if (defined $opts{d}){
		$database_name = $opts{d};
	}
	if (defined $opts{e}){
		$enzymeName = $opts{e};
	}
	if (defined $opts{b}){
		$doBinningBit = $opts{b};
	}
	if (defined $opts{f}){
		$DBFiletype = $opts{f};
	}
}
else {
	$commandlineSwitch =0;
}

#Determine operating system.
my $perlversion = $^O;
if ($perlversion eq "MSWin32"){
	use Win32::FileOp qw(ShellExecute);
}

#GUI variables
my (
	$Form1,
	$mmenubar,
	@mainmenu,
	$lbOutput,
	$lbInput,
	$lbTolerance,
	$lbToleranceProtein,
	$lbEnzyme,
	$lbAASelection,
	$edInput,
	$edOutput,
	$edProLower,
	$edProUpper,
	$edPepLower,
	$edPepUpper,
	$edRetentionTimeA,
	$edRetentionTimeB,
	$cbBinning,
	$rbPPM,
	$rbDa,
	$rbMMU,
	$edToleranceValue,
	$btAllAA,
	$btProcess,
	$cbA,
	$cbC,
	$cbD,
	$cbE,
	$cbF,
	$cbG,
	$cbH,
	$cbI,
	$cbK,
	$cbL,
	$cbM,
	$cbN,
	$cbP,
	$cbQ,
	$cbR,
	$cbS,
	$cbT,
	$cbV,
	$cbW,
	$cbY,
	$btOutputSelect,
	$btDBSelect,
	$lbDBFiletype,
	$lboxpepTolerance,
	$lboxProteinTolerance,
	$lboxEnzyme,
	$lbpepTolerance,
	$lbToleranceValue,
	$lbTolerancePep,
	$lbProteinTolerance,
	$lbPepLower,
	$lbPepUpper,
	$lbProLower,
	$lbProUpper,
	$lbRetentionTime,
	$lbRetentionTimeA,
	$lbRetentionTimeB,
	$rbPPM1,
	$rbDa1,
	$rbMMU1,
	$omDBFiletype,
	#$LoadDialogFolder,
	#$LoadDialogFile,
	);

# Gobal variables
my (
	$outputPath,
	$currentPath,
	%AASelect,
	@AAlist,
	$AA,
	%Output_filename_proteins_hash,
	%Output_filename_hash,
	%Output_filename_peptides_hash,
	$toleranceType,
	@toleranceValue,
	$toleranceTypePro,
	@toleranceValuePro,
	$lower_mass_pep,
	$endmass_pep,
	$lower_mass_pro,
	$endmass_pro,
	@DBFiletypeOptions,
	%InAAProtein_hash,
	%InAAPeptide_hash,
	%InAAConPeptide_hash,
	$Hydro_a,
	$Hydro_b
   );
#old variables, 
my ($cleaveCTerm, $cleaveNTerm, $cleaveCTermException, $cleaveNTermException);
my (@cleaveCTerm_array,@cleaveNTerm_array, @cleaveCTermException_array, @cleaveNTermException_array);


#Set default values
$doBinningBit = 1;
$toleranceType = "ppm";
@toleranceValue = qw(0.1 0.5 1 5 10 15 20 25 30 40 50 75 100 200 250 300 400 500 750 1000);
$toleranceTypePro = "Da";
@toleranceValuePro = qw(0.1 0.5 1 5 10 15 20 25 30 40 50 75 100 200 250 300 400 500 600 700 750 800 900 1000 5000);
%AASelect = qw(A 0 C 0 D 0 E 0 F 0 G 0 H 0 I 0 K 0 L 0 M 0 N 0 P 0 Q 0 R 0 S 0 T 0 V 0 W 0 Y 0 );
@DBFiletypeOptions = ("NCBInr_Fasta", "Swissprot_Fasta", "IPI_Fasta", "other");
if ($commandlineSwitch ==0){
	$DBFiletype = $DBFiletypeOptions[0];
}
$currentPath = "c:/";
$outputPath = "c:/";
$database_name = "c:/";
$lower_mass_pro = "1000";
$endmass_pro = "35000";
$lower_mass_pep = "50";
$endmass_pep = "4000";
$Hydro_a = "10.8";
$Hydro_b = "0.386";
#make gui if we are in gui mode
if ($commandlineSwitch ==0){
	$Form1 = MainWindow->new;
	$Form1->title("ProteomeXplorer");
	$Form1->geometry("582x378+8+8");
	
	#($LoadDialogFolder) = $Form1->FileDialog(-Title =>'Select a folder',-Create => 0);
	#$LoadDialogFolder->configure(-FPat => '*',-ShowAll => 'Yes', -Path => "c:/", -SelDir => 1);
	#$Horiz = 1;
	#$grab = 1;
	
	#($LoadDialogFile) = $Form1->FileDialog(-Title =>'Select a file',-Create => 0);
	#$LoadDialogFile->configure(-FPat => '*',-ShowAll => 'Yes', -Path => "c:/");
	#$Horiz = 1;

	$mmenubar = $Form1->Frame(-relief => 'raised', -borderwidth => 2, -height => 16)
	->pack(-side => 'top', -anchor => 'n', -expand => 1, -fill => 'x');
	push (@mainmenu, $mmenubar->Menubutton(-text => "File", -tearoff => 0, -menuitems => [ 
		  ["command" => "Open Database", -font => "Arial 8 normal", -command => \&selectDatabase],
		  ["command" => "Select output folder", -font => "Arial 8 normal", -command => \&outputDirectory],
		  ["command" => "Digest Database", -font => "Arial 8 normal", -command => \&digestDB],
		  ["command" => "Calculate AMT's", -font => "Arial 8 normal", -command => \&AMTwindow],
		  ["command" => "Quit", -font => "Arial 8 normal", -command =>sub{exit}]
	]));
	$mainmenu[0]->pack(-side => 'left');
	push (@mainmenu, $mmenubar->Menubutton(-text => "Help", -tearoff => 0, -menuitems => [ 
		  ["command" => "Help", -font => "Arial 8 normal", -command => \&help],
		  ["command" => "About", -font => "Arial 8 normal", -command => \&about]
	]));
	$mainmenu[1]->pack(-side => 'left');
	$lbOutput = $Form1->Label(
                          -text => "Output Location", 
                          -background => "#D4D0C8", 
                          -cursor => "", 
                          -font => "Tahoma 8 normal"
                          );

	
	$lbOutput->place(-width => 79, 
					 -height => 13, 
					 -x => 8, 
					 -y => 48);
	
	$lbInput = $Form1->Label(
							 -text => "Database file", 
							 -background => "#D4D0C8", 
							 -cursor => "", 
							 -font => "Tahoma 8 normal"
							 );
	
	
	$lbInput->place(-width => 79, 
					-height => 13, 
					-x => 8, 
					-y => 80);
	
	$lbTolerance = $Form1->Label(
								 -text => "Tolerance type", 
								 -background => "#D4D0C8", 
								 -cursor => "", 
								 -font => "Tahoma 8 normal"
								 );
	
	
	$lbTolerance->place(-width => 79, 
						-height => 13, 
						-x => 16, 
						-y => 136);
	
	$lbTolerancePep = $Form1->Label(
									  -text => "Tolerance Value Peptide", 
									  -background => "#D4D0C8", 
									  -cursor => "", 
									  -font => "Tahoma 8 normal"
									  );
	
	
	$lbTolerancePep->place(-width => 127, 
							 -height => 13, 
							 -x => 176, 
							 -y => 112);
	
	$lbEnzyme = $Form1->Label(
							  -text => "Enzyme", 
							  -background => "#D4D0C8", 
							  -cursor => "", 
							  -font => "Tahoma 8 normal"
							  );
	
	
	$lbEnzyme->place(-width => 55, 
					 -height => 13, 
					 -x => 8, 
					 -y => 232);
	
	$lbAASelection = $Form1->Label(
								   -text => "AA reports", 
								   -background => "#D4D0C8", 
								   -cursor => "", 
								   -font => "Tahoma 8 normal"
								   );
	
	
	$lbAASelection->place(-width => 63, 
						  -height => 13, 
						  -x => 336, 
						  -y => 224);
	
	$lbDBFiletype = $Form1->Label(
								  -text => "Database file type", 
								  -background => "#D4D0C8", 
								  -cursor => "", 
								  -font => "Tahoma 8 normal"
								  );
	
	
	$lbDBFiletype->place(-width => 103, 
						 -height => 13, 
						 -x => 264, 
						 -y => 80);
	
	$omDBFiletype = $Form1->Optionmenu(
                         -textvariable =>\$DBFiletype,
                        -options => [@DBFiletypeOptions],
                         -font => "Tahoma 8 normal"
                         );

	$omDBFiletype->place(-width => 120, 
					 -height => 21, 
					 -x => 370, 
					 -y => 74);

	
	$lbToleranceValue = $Form1->Label(
									-text => "Tolerance Value Protein", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);
	
	$lbToleranceValue->place(-width => 119, 
						   -height => 13, 
						   -x => 370, 
						   -y => 112);
	
	$lbPepLower = $Form1->Label(
									-text => "Lower", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);	
	
	$lbPepLower->place(-width => 40, 
						   -height => 13, 
						   -x => 290, 
						   -y => 135);
	
	$lbPepUpper = $Form1->Label(
									-text => "Upper", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);
	
	$lbPepUpper->place(-width => 40, 
						   -height => 13, 
						   -x => 290, 
						   -y => 170);
	
	$lbProLower = $Form1->Label(
									-text => "Lower", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);	
	
	$lbProLower->place(-width => 40, 
						   -height => 13, 
						   -x => 490, 
						   -y => 135);
	
	$lbProUpper = $Form1->Label(
									-text => "Upper", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);
	
	$lbProUpper->place(-width => 40, 
						   -height => 13, 
						   -x => 490, 
						   -y => 170);
	
	$lbRetentionTime = $Form1->Label(
									-text => "Retention time parameters:", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);
	
	$lbRetentionTime->place(-width => 140, 
						   -height => 13, 
						   -x => 250, 
						   -y => 350);
	
	$lbRetentionTimeA = $Form1->Label(
									-text => "a", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);
	
	$lbRetentionTimeA->place(-width => 10, 
						   -height => 13, 
						   -x => 405, 
						   -y => 350);
	
	$lbRetentionTimeB = $Form1->Label(
									-text => "b", 
									-background => "#D4D0C8", 
									-cursor => "", 
									-font => "Tahoma 8 normal"
									);
	
	$lbRetentionTimeB->place(-width => 10, 
						   -height => 13, 
						   -x => 485, 
						   -y => 350);
	
	$edInput = $Form1->Entry(
							 -cursor => "", 
							 -font => "Tahoma 8 normal",
							 -textvariable => \$database_name,
							 );
	
	
	$edInput->place(-width => 121, 
					-height => 21, 
					-x => 88, 
					-y => 72);
	
	$edOutput = $Form1->Entry(
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -textvariable => \$outputPath,
							  );
	
	
	$edOutput->place(-width => 121, 
					 -height => 21, 
					 -x => 88, 
					 -y => 40);
	
	$edPepLower = $Form1->Entry(
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -textvariable => \$lower_mass_pep
							  );
	
	
	$edPepLower->place(-width => 60, 
					 -height => 21, 
					 -x => 290, 
					 -y => 150);
	
	$edPepUpper = $Form1->Entry(
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -textvariable => \$endmass_pep,
							  );
	
	
	$edPepUpper->place(-width => 60, 
					 -height => 21, 
					 -x => 290, 
					 -y => 185);
	
	$edProLower = $Form1->Entry(
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -textvariable => \$lower_mass_pro,
							  );
	
	
	$edProLower->place(-width => 60, 
					 -height => 21, 
					 -x => 490, 
					 -y => 150);
	
	$edProUpper = $Form1->Entry(
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -textvariable => \$endmass_pro,
							  );
	
	
	$edProUpper->place(-width => 60, 
					 -height => 21, 
					 -x => 490, 
					 -y => 185);
	
	$edRetentionTimeA = $Form1->Entry(
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -textvariable => \$Hydro_a,
							  );
	
	
	$edRetentionTimeA->place(-width => 60, 
					 -height => 21, 
					 -x => 420, 
					 -y => 344);
	
	$edRetentionTimeB = $Form1->Entry(
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -textvariable => \$Hydro_b,
							  );
	
	
	$edRetentionTimeB->place(-width => 60, 
					 -height => 21, 
					 -x => 500, 
					 -y => 344);
	
	$cbBinning = $Form1->Checkbutton(
							-text => "Generate Binned mass files",
							-variable => \$doBinningBit
									 );
	
	
	$cbBinning->place(-width => 160, 
					  -height => 17, 
					  -x => 10, 
					  -y => 112);
	
	$rbPPM = $Form1->Radiobutton(-text => "ppm",
								 -value => "ppm",
								 -variable => \$toleranceType,
								 );
	
	
	$rbPPM->place(-width => 41, 
				  -height => 17, 
				  -x => 163, 
				  -y => 136);
	
	$rbDa = $Form1->Radiobutton(-text => "Da",
								 -value => "Da",
								 -variable => \$toleranceType,
								);
	
	
	$rbDa->place(-width => 41, 
				 -height => 17, 
				 -x => 160, 
				 -y => 152);
	
	$rbMMU = $Form1->Radiobutton(
								 -text => "mmu",
								 -value => "mmu",
								 -variable => \$toleranceType,
								 );
	
	
	$rbMMU->place(-width => 41, 
				  -height => 17, 
				  -x => 164, 
				  -y => 168);
	
	my $sbEnzyme = $Form1->Scrollbar();
	$lboxEnzyme = $Form1->Listbox(
								-cursor => "", 
								-font => "Tahoma 8 normal",
								-selectmode => "single",
								-exportselection =>0,
								-yscrollcommand=>['set' => $sbEnzyme]
								);
	
	
	$lboxEnzyme->place(-width => 121, 
					 -height => 97, 
					 -x => 64, 
					 -y => 232);
	$sbEnzyme->configure(-command => ['yview' => $lboxEnzyme]);
	$sbEnzyme->place(-x => 182, -y => 232, -height => 97, -width => 16);
	
	
	$cbA = $Form1->Checkbutton(-text => "A",
							-variable => \$AASelect{"A"}
							   );
	
	
	$cbA->place(-width => 33, 
				-height => 17, 
				-x => 264, 
				-y => 248);
	
	$cbC = $Form1->Checkbutton(
							   -text => "C",
							-variable => \$AASelect{"C"}
							   );
	
	
	$cbC->place(-width => 33, 
				-height => 17, 
				-x => 264, 
				-y => 264);
	
	$cbD = $Form1->Checkbutton(
							   -text => "D",
							-variable => \$AASelect{"D"}
							   );
	
	$cbD->place(-width => 33, 
				-height => 17, 
				-x => 264, 
				-y => 280);
	
	$cbE = $Form1->Checkbutton(
							   -text => "E",
							-variable => \$AASelect{"E"}
							   );
	
	
	$cbE->place(-width => 33, 
				-height => 17, 
				-x => 264, 
				-y => 296);
	
	$cbF = $Form1->Checkbutton(
							   -text => "F",
							-variable => \$AASelect{"F"}
							   );
	
	
	$cbF->place(-width => 33, 
				-height => 17, 
				-x => 264, 
				-y => 312);
	
	$cbG = $Form1->Checkbutton(
							   -text => "G",
							-variable => \$AASelect{"G"}
							   );
	
	
	$cbG->place(-width => 33, 
				-height => 17, 
				-x => 320, 
				-y => 248);
	
	$cbH = $Form1->Checkbutton(-text => "H",
							-variable => \$AASelect{"H"}
							   );
	
	
	$cbH->place(-width => 33, 
				-height => 17, 
				-x => 320, 
				-y => 264);
	
	$cbI = $Form1->Checkbutton(
							   -text => "I",
							-variable => \$AASelect{"I"}
							   );
	
	
	$cbI->place(-width => 33, 
				-height => 17, 
				-x => 320, 
				-y => 280);
	
	$cbK = $Form1->Checkbutton(
							   -text => "K",
							-variable => \$AASelect{"K"}
							   );
	
	
	$cbK->place(-width => 33, 
				-height => 17, 
				-x => 320, 
				-y => 296);
	
	$cbL = $Form1->Checkbutton(
							   -text => "L",
							-variable => \$AASelect{"L"}
							   );
	
	
	$cbL->place(-width => 33, 
				-height => 17, 
				-x => 320, 
				-y => 312);
	
	$cbM = $Form1->Checkbutton(
							   -text => "M",
							-variable => \$AASelect{"M"}
							   );
	
	
	$cbM->place(-width => 33, 
				-height => 17, 
				-x => 376, 
				-y => 248);
	
	$cbN = $Form1->Checkbutton(
							   -text => "N",
							-variable => \$AASelect{"N"}
							   );
	
	
	$cbN->place(-width => 33, 
				-height => 17, 
				-x => 376, 
				-y => 264);
	
	$cbP = $Form1->Checkbutton(
							   -text => "P",
							-variable => \$AASelect{"P"}
							   );
	
	
	$cbP->place(-width => 33, 
				-height => 17, 
				-x => 376, 
				-y => 280);
	
	$cbQ = $Form1->Checkbutton(
							   -text => "Q",
							-variable => \$AASelect{"Q"}
							   );
	
	
	$cbQ->place(-width => 33, 
				-height => 17, 
				-x => 376, 
				-y => 296);
	
	$cbR = $Form1->Checkbutton(
							   -text => "R",
							-variable => \$AASelect{"R"}
							   );
	
	
	$cbR->place(-width => 33, 
				-height => 17, 
				-x => 376, 
				-y => 312);
	
	$cbS = $Form1->Checkbutton(
							   -text => "S",
							-variable => \$AASelect{"S"}
							   );
	
	
	$cbS->place(-width => 33, 
				-height => 17, 
				-x => 424, 
				-y => 248);
	
	$cbT = $Form1->Checkbutton(
							   -text => "T",
							-variable => \$AASelect{"T"}
							   );
	
	
	$cbT->place(-width => 33, 
				-height => 17, 
				-x => 424, 
				-y => 264);
	
	$cbV = $Form1->Checkbutton(
							   -text => "V",
							-variable => \$AASelect{"V"}
							   );
	
	
	$cbV->place(-width => 33, 
				-height => 17, 
				-x => 424, 
				-y => 280);
	
	$cbW = $Form1->Checkbutton(
							   -text => "W",
							-variable => \$AASelect{"W"}
							   );
	
	
	$cbW->place(-width => 33, 
				-height => 17, 
				-x => 424, 
				-y => 296);
	
	$cbY = $Form1->Checkbutton(
							   -text => "Y",
							-variable => \$AASelect{"Y"}
							   );
	
	
	$cbY->place(-width => 33, 
				-height => 17, 
				-x => 424, 
				-y => 312);
	
	$btAllAA = $Form1->Button(
							  -text => "Select all AA's", 
							  -cursor => "", 
							  -font => "Tahoma 8 normal",
							  -underline => 11,
							 -command => \&selectAllAA
							  );
	
	
	$btAllAA->place(-width => 75, 
					-height => 25, 
					-x => 480, 
					-y => 248);
	
	$btProcess = $Form1->Button(
								-text => "Digest Database", 
								-cursor => "", 
								-font => "Tahoma 8 normal",
								-underline => 0,
								-command => \&digestDB
								);
	
	
	$btProcess->place(-width => 90, 
					  -height => 25, 
					  -x => 64, 
					  -y => 344);
	
	$btOutputSelect = $Form1->Button(
									 -text => "..", 
									 -cursor => "", 
									 -font => "Tahoma 8 normal",
									-command => \&outputDirectory
									 );
	
	
	$btOutputSelect->place(-width => 19, 
						   -height => 25, 
						   -x => 224, 
						   -y => 40);
	
	$btDBSelect = $Form1->Button(
								 -text => "..", 
								 -cursor => "", 
								 -font => "Tahoma 8 normal",
								-command => \&selectDatabase,
								 );
	
	
	$btDBSelect->place(-width => 19, 
					   -height => 25, 
					   -x => 224, 
					   -y => 72);
	
	my $sbPepTolerance = $Form1->Scrollbar();
	$lboxpepTolerance = $Form1->Listbox(
									  -cursor => "", 
									  -font => "Tahoma 8 normal",
									  -selectmode => "extended",
									  -exportselection =>0,
									  -yscrollcommand=>['set' => $sbPepTolerance]
									  );
	$lboxpepTolerance->place(-width => 41, 
						   -height => 65, 
						   -x => 220, 
						   -y => 136);
	$sbPepTolerance->configure(-command => ['yview' => $lboxpepTolerance]);
	$sbPepTolerance->place(-x => 260, -y => 136, -height => 65, -width => 16);
	
	my $sbProTolerance = $Form1->Scrollbar();	
	$lboxProteinTolerance = $Form1->Listbox(
										  -cursor => "", 
										  -font => "Tahoma 8 normal",
										  -selectmode => "extended",
										  -exportselection =>0,
										  -yscrollcommand=>['set' => $sbProTolerance]
										  );
	
	
	$lboxProteinTolerance->place(-width => 41, 
							   -height => 65, 
							   -x => 420, 
							   -y => 136);
	$sbProTolerance->configure(-command => ['yview' => $lboxProteinTolerance]);
	$sbProTolerance->place(-x => 460, -y => 136, -height => 65, -width => 16);
	
	$rbPPM1 = $Form1->Radiobutton(
								  -text => "ppm",
								 -value => "ppm",
								 -variable => \$toleranceTypePro,
								  );
	
	
	$rbPPM1->place(-width => 42, 
				   -height => 17, 
				   -x => 373, 
				   -y => 136);
	
	$rbDa1 = $Form1->Radiobutton(
								 -text => "Da",
								 -value => "Da",
								 -variable => \$toleranceTypePro,
								 );
	
	
	$rbDa1->place(-width => 41, 
				  -height => 17, 
				  -x => 370, 
				  -y => 152);
	
	$rbMMU1 = $Form1->Radiobutton(
								  -text => "mmu",
								 -value => "mmu",
								 -variable => \$toleranceTypePro,
								  );
	
	
	$rbMMU1->place(-width => 41, 
				   -height => 17, 
				   -x => 374, 
				   -y => 168);
}

my $enzymeConfigFile = "enzyme.xml";
#Generate enzyme object from xml config file
my $enzyme = XMLin($enzymeConfigFile);

#access data arra, generate list of databases
my @enzymeNames =sort (keys %{$enzyme->{'enzyme'}});
	
#if we are running gui mode start
if ($commandlineSwitch ==0){
	#fill list boxes
	$lboxpepTolerance->insert('end', @toleranceValue);
	$lboxpepTolerance->selectionSet(4);
	$lboxProteinTolerance->insert('end', @toleranceValuePro);
	$lboxProteinTolerance->selectionSet(10);
	$lboxEnzyme->insert('end', @enzymeNames);
	$lboxEnzyme->selectionSet(0);
	#$Form1->bind('<KeyRelease-d>' => \&digestDB);
	#$Form1->bind('<KeyRelease-a>' => \&selectAllAA);
	#$Form1->bind('<KeyRelease-A>' => \&selectAllAA);
	#$Form1->bind('<KeyRelease-x>' => \&quitProgram);
	MainLoop;
}
#else run from the command line
elsif ($commandlineSwitch ==1) {
	&selectAllAA;
	&digestDB;
}

############################ SUBS ############################



#######################
#selectDatabase Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub selectDatabase {
	#r:/Dev/ProteomeXplorer/10proteins.txt
	#$database_name = "10proteins.txt";
	my ($LoadDialogFile) = $Form1->FileDialog(-Title =>'Select a file',-Create => 0);
	my $Horiz = 1;
	
	$LoadDialogFile->configure(-FPat => '*',-ShowAll => 'YES', -Path => $currentPath);
    $database_name= $LoadDialogFile->Show(-Horiz => $Horiz);
    if (!defined($database_name)) {
        $database_name = "Fine,Cancel, but no Chdir anymore!!!";
        $LoadDialogFile->configure(-Chdir =>'NO');
    }
	return ();
}

#######################
#outputDirectory Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub outputDirectory {
	my ($LoadDialogFolder) = $Form1->FileDialog(-Title =>'Select a folder',-Create => 0);
	$LoadDialogFolder->configure(-FPat => '*',-ShowAll => 'Yes', -Path => "c:/", -SelDir => 1);
	my $Horiz = 1;
	my $grab = 1;
	$outputPath = $LoadDialogFolder->Show(-Horiz => $Horiz, -Grab => $grab, -Path => $currentPath);
    if (!defined($outputPath)) {
        $outputPath = "Fine,Cancel, but no Chdir anymore!!!";
        $LoadDialogFolder->configure(-Chdir =>'NO');
    }
	$currentPath = $outputPath;
	return ();
}

#######################
#selectAMTfile Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub selectAMTfile {
	my ($LoadDialogAMT) = $Form1->FileDialog(-Title =>'Select a file',-Create => 0);
	$LoadDialogAMT->configure(-FPat => '*',-ShowAll => 'Yes', -Path => $currentPath);
	my $Horiz = 1;
    my $filenameAMT = $LoadDialogAMT->Show(-Horiz => $Horiz);
    if (!defined($filenameAMT)) {
        $filenameAMT = "Fine,Cancel, but no Chdir anymore!!!";
        $LoadDialogAMT->configure(-Chdir =>'NO');
    }
	return ($filenameAMT);
}

#######################
#selectAllAA Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub selectAllAA {
	foreach my $TempAA (keys %AASelect){
		$AASelect{$TempAA} = 1;
	}
	if ($commandlineSwitch ==1){
		$Form1->update;
	}
	return ();
}
#######################
#digestDB Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub digestDB {
	#Grab starting conditions from GUI
	#Build an AA list
	foreach $AA(keys %AASelect){
		if ($AASelect{$AA} == 1){
			push (@AAlist, $AA)
		}
	}
	my ($tempTolerance,
		@selectedProTolerance,
		@selectedPepTolerance,
		$selectedEnzyme,
		@database_name_split,
		$Output_filename,
		$Output_filename_peptides,
		$Output_filename_proteins,
		$Output_filename_CAFpeptides,
		%Output_filename_hash,
		%Output_filename_peptides_hash,
		%Output_filename_proteins_hash,
		$fh,
		$fh_pep,
		$fh_pro,
		%Output_filename_list,
		$AATotalhash_ref,
		$TotalNoOfPeptides,
		$TotalNoOfAAPeptides,
		$NoOfAAinDatabase,
		$NoOfAllAAinDatabase,
		$NoAAPeptides,
		%AA_Counts,
		%AATotalhash,
		%peptidearrayReference,
		$index_no,
		$processProteinFlag,
		$inprotein,
		$total_AA_ratio_temp,
		$AAtemppercent,
		$protein,
		@Accession_list,
		$AcessionNo,
		$tempAcessionNo,
		$tempProtein,
		$CAFTotal,
		$line,
		$mointor,
		$AAvalue,
		@CompleteAAlist,
		%binFileList,
	);
	#initalise parameters
	$TotalNoOfPeptides = 0;
	$TotalNoOfAAPeptides = 0;
	$NoOfAAinDatabase = 0;
	$NoOfAllAAinDatabase = 0;
	$NoAAPeptides = 0;
	$index_no = 0;
	$processProteinFlag = 0;
	$inprotein=0;
	$tempAcessionNo = 0;
	#Build binning list - values can be indexed to original list when we need to use them
	@selectedProTolerance = $lboxProteinTolerance->curselection();
	@selectedPepTolerance = $lboxpepTolerance->curselection();
	#Determine enzyme name
	$selectedEnzyme = $lboxEnzyme->curselection();
	$enzymeName = $enzymeNames[$selectedEnzyme];
	#Set output file names
	@database_name_split =  split (/\./,$database_name);
	$Output_filename = $database_name_split[0] . $database_name_split[1] . "_" . $enzymeName ."_All_list.txt";
	$Output_filename_peptides = $database_name_split[0] . $database_name_split[1] . "_" . $enzymeName . "_All_peptides.txt";
	$Output_filename_proteins = $database_name_split[0] . $database_name_split[1] . "_" . $enzymeName . "_All_proteins.txt";
	#Output 
	#open output file
	open (OUTPUT, ">$Output_filename") or die "can't open output file";
	open (OUTPUTPEP, ">$Output_filename_peptides") or die "can't open peptide output file";
	open (OUTPUTPRO, ">$Output_filename_proteins") or die "can't open protein output file";
	$binFileList{$Output_filename_peptides}="0";
	$binFileList{$Output_filename_proteins}="1";
	foreach $AA (@AAlist ){
		$Output_filename = $database_name_split[0] . $database_name_split[1] . "_" . $enzymeName  . "_" . $AA . "_list.txt";
		$Output_filename_list{$AA} = $Output_filename;
		$Output_filename_peptides = $database_name_split[0] . $database_name_split[1] . "_" . $enzymeName . "_" . $AA .  "_peptides.txt";
		$Output_filename_proteins = $database_name_split[0] . $database_name_split[1] . "_" . $enzymeName  . "_" . $AA .  "_proteins.txt";
		$binFileList{$Output_filename_peptides}="0";
		$binFileList{$Output_filename_proteins}="1";
		local *FILE;
		local *FILE2;
		local *FILE3;
		open( FILE, ">$Output_filename" ) or die( "Bad things happened : $!" );
		open( FILE2, ">$Output_filename_peptides" ) or die( "Bad things happened : $!" );
		open( FILE3, ">$Output_filename_proteins" ) or die( "Bad things happened : $!" );
		$Output_filename_hash{$AA} = *FILE;
		$Output_filename_peptides_hash{$AA} = *FILE2;
		$Output_filename_proteins_hash{$AA} = *FILE3;
	}
	
	
	foreach $AA (@AAlist){
		$AATotalhash{$AA} = 0;
	}
	$AATotalhash_ref = \%AATotalhash;
	#copy enzyme paramters to temp variables from enzyme object
	$cleaveCTerm = $enzyme->{'enzyme'}->{$enzymeName}->{'cleaveCTerminal'}->{'cleaveAA'};
	$cleaveNTerm = $enzyme->{'enzyme'}->{$enzymeName}->{'cleaveNTerminal'}->{'cleaveAA'};
	$cleaveCTermException = $enzyme->{'enzyme'}->{$enzymeName}->{'cleaveCTerminal'}->{'cleaveException'};
	$cleaveNTermException = $enzyme->{'enzyme'}->{$enzymeName}->{'cleaveNTerminal'}->{'cleaveException'};
	#split cleavge AA's in to arrays
	if (defined $cleaveCTerm){
		@cleaveCTerm_array = (split //,$cleaveCTerm);
	}
	if (defined $cleaveNTerm){
		@cleaveNTerm_array = (split //,$cleaveNTerm);
	}
	if (defined $cleaveCTermException){
		@cleaveCTermException_array = (split //,$cleaveCTermException);
	}
	if (defined $cleaveNTermException){	
		@cleaveNTermException_array = (split //,$cleaveNTermException);
	}
	
	#and read in sequences
	#foreach line in the database
	open (DATABASE, "$database_name")||die "cant open database\n";
	print "Digesting Database\n";
	while (<DATABASE>){
		$line = $_;
		chomp $line;
		#if line starts with a ">" then its the comment line of a new protein entry 
		if (/^\>/){
			$AcessionNo = $tempAcessionNo;
			#Save acession number as key and sequence as value
			@Accession_list = split (/\|/, $line);
			#add in parsing for different database formats
			if ($DBFiletype eq "NCBInr_Fasta"){
				#>gi|7290020|gb|AAF45487.1| (AE003417) EG:BACR37P7.2 gene product [Drosophila melanogaster]
				#for NCBInr database
				$tempAcessionNo = $Accession_list[1];
				
			}
			elsif ($DBFiletype eq "Swissprot_Fasta"){
				#>gi|135014|sp|P27366|SUBI_SYNP7 SULFATE-BINDING PROTEIN PRECURSOR
				# for swissprot number
				($tempAcessionNo) = $Accession_list[3];
			}
			elsif ($DBFiletype eq "IPI_Fasta"){
				#>IPI:IPI00387610.1|ENSEMBL:ENSRNOP00000030672 Tax_Id=10116 Ensembl_locations(Chr-bp):1-66881881
				#for ipi database
				($tempAcessionNo) = substr ($Accession_list[0], 5, 15);
			}
			else {
				#>135014|something|27366|
				#for gi number - differnet comment line format
				($tempAcessionNo) = substr ($Accession_list[0], 1, (length $Accession_list[0]));
			}
			$inprotein =1;
		}
		elsif ($inprotein ==1){
			#first line of sequence
			$inprotein =0;
			if ($index_no == 0){
				$index_no++;
				$tempProtein = $line;
				next;
			}
			
			$protein = $tempProtein;
			#$AcessionNo = $tempAcessionNo;
			$processProteinFlag = 1;
			$tempProtein = $line;
		}
		else {
			#just a normal line, entry alrady made into %seqhash
			#add sequence to entry
			$tempProtein = (  $tempProtein . $line);
			next;
		}
		if ($processProteinFlag ==1){
			$processProteinFlag =0;
			($NoOfAllAAinDatabase, $TotalNoOfPeptides, $AATotalhash_ref) = &processProtein ($protein, $index_no, $enzymeName, $AcessionNo, $NoOfAllAAinDatabase, $TotalNoOfPeptides, \%Output_filename_proteins_hash, \%Output_filename_peptides_hash, \%Output_filename_hash, $AATotalhash_ref );
			$index_no++;
			#%AATotalhash = %{$AATotalhash_ref};
			#print the current index number is it is divisable by 100
			$mointor = $index_no/100;
			#$mointor = number_split ($mointor, 2);
			unless ($mointor =~ /\d+\.\d+/){
				print "$index_no\n";
			}
		} 	
	}
	$protein = $tempProtein;
	$AcessionNo = $tempAcessionNo;
	($NoOfAllAAinDatabase, $TotalNoOfPeptides, $AATotalhash_ref) = &processProtein ($protein, $index_no, $enzymeName, $AcessionNo, $NoOfAllAAinDatabase, $TotalNoOfPeptides, \%Output_filename_proteins_hash, \%Output_filename_peptides_hash, \%Output_filename_hash, $AATotalhash_ref );
	print "\n";
	print OUTPUT "\n\nDatabase used:\t$database_name\n";
	print OUTPUT "\nEnzyme:\t$enzymeName\n";
	print OUTPUT "Totals in the database\n";
	print OUTPUT "No. of Amino Acids\t$NoOfAllAAinDatabase\n";
	#print OUTPUT "No. that are $AA\t$NoOfAAinDatabase\n";
	print OUTPUT "Peptides\t$TotalNoOfPeptides\n";
	#print OUTPUT "$AA containing Peptides\t$TotalNoOfAAPeptides\n";
	print OUTPUT "No. of Proteins\t$index_no\n\n";
	print OUTPUT "AA\tCount\t\%\n";
	
	#master list do for all AA
	@CompleteAAlist = &aa_list;
	foreach $AAvalue (@CompleteAAlist){
		if ($AATotalhash_ref->{$AAvalue}){
			$AAtemppercent = ($AATotalhash_ref->{$AAvalue}/$NoOfAllAAinDatabase) *100;
		}
		else {
			$AAtemppercent = 0;
		}
		
		$AAtemppercent = number_split ($AAtemppercent, 2);
		print OUTPUT "$AAvalue\t$AATotalhash_ref->{$AAvalue}\t$AAtemppercent\n";
	}
	foreach $AA (@AAlist){
		$fh = $Output_filename_hash{$AA};
		print $fh "\n\nDatabase used:\t$database_name\n";
		print $fh "\nEnzyme:\t$enzymeName\n";
		print $fh "Totals in the database\n";
		print $fh "No. of Amino Acids\t$NoOfAllAAinDatabase\n";
		print $fh "No. that are $AA\t$AATotalhash_ref->{$AA}\n";
		print $fh "Peptides\t$TotalNoOfPeptides\n";
		#print $fh "$AA containing Peptides\t$TotalNoOfAAPeptides\n";
		print $fh "No. of Proteins\t$index_no\n\n";
		$total_AA_ratio_temp = ($AATotalhash_ref->{$AA}/$NoOfAllAAinDatabase)*100;
		$total_AA_ratio_temp = number_split ($total_AA_ratio_temp, 2);
	}

	#close output files
	close OUTPUT|| warn "Couldn't close file $Output_filename\n";	
	close OUTPUTPEP|| warn "Couldn't close file $Output_filename_peptides\n";
	close OUTPUTPRO|| warn "Couldn't close file $Output_filename_proteins\n";
	foreach $AA (@AAlist){
		$fh_pro = $Output_filename_proteins_hash{$AA};
		$fh = $Output_filename_hash{$AA};
		$fh_pep = $Output_filename_peptides_hash{$AA};
		close $fh or warn "could not close output file: $!";
		close $fh_pep or warn "could not close peptide file: $!";
		close $fh_pro or warn "could not close protein file: $!";
	}
		
	my (@StatFileNames) = &aa_analysis (\%Output_filename_list);
	
	print "Binning peptide and protein mass files\n";
	my ($currentfile, $tolerance, $tolerance_type);
	if ($doBinningBit ==1){
		#if we are in GUI mode
		if ($commandlineSwitch ==0){
			foreach $currentfile (keys %binFileList){
				print "Binning $currentfile\n";
				if ($binFileList{$currentfile} == 0){
					foreach $tempTolerance (@selectedPepTolerance){
						#do binning as per GUI parameters
						$tolerance = $toleranceValue[$tempTolerance];
						$tolerance_type = $toleranceType;
						BinMass ($currentfile, $tolerance_type, $tolerance, $lower_mass_pep, $endmass_pep);
					}
				}
				else {
					foreach $tempTolerance (@selectedProTolerance){
						#do binning as per GUI parameters
						$tolerance = $toleranceValuePro[$tempTolerance];
						$tolerance_type = $toleranceTypePro;
						BinMass ($currentfile, $tolerance_type, $tolerance, $lower_mass_pro, $endmass_pro);
					}
				}
			}
		}
		else {
			@selectedPepTolerance = qw(0.1 1 10 50 100);
			@selectedProTolerance = qw(50);
			foreach $currentfile (keys %binFileList){
				print "Binning $currentfile\n";
				if ($binFileList{$currentfile} == 0){
					foreach $tempTolerance (@selectedPepTolerance){
						#do binning as per GUI parameters
						$tolerance = $tempTolerance;
						$tolerance_type = "ppm";
						BinMass ($currentfile, $tolerance_type, $tolerance, $lower_mass_pep, $endmass_pep);
					}
				}
				else {
					foreach $tempTolerance (@selectedProTolerance){
						$tolerance = $tempTolerance;
						$tolerance_type = "Da";
						BinMass ($currentfile, $tolerance_type, $tolerance, $lower_mass_pro, $endmass_pro);
					}
				}
			}
		}
	}
	&generateStats(@StatFileNames);
	return ();
}

#######################
#number_split Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub number_split {
	my ( $number, $deciminal_places ) = @_;
	my @numsplit =  split (/\./,$number);
	if ($#numsplit == 1){
		$number = $numsplit[0] . "." . substr($numsplit[1],0,$deciminal_places);
	}
	else {
		$number = $numsplit[0];
	}
	return $number;
}

#######################
#BinMass Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub BinMass {
	#Input
	#input Filename 
	my ($filename, $tolerance_type, $tolerance_value, $lower_mass, $endmass) = @_;
	my (@mass_array,$line,
		$newfilename,
		@filesplit,
		$upper_mass,
		$graph_value,
		$bin,
		$mass_window,
		$i,
		$mass_out,
		$mass,
		$unique_percent,
	);

	@filesplit =  split (/\_/,$filename);
	$newfilename = $filesplit[0] . "_" . $filesplit[1] . "_" . $filesplit[2]. "_" .$tolerance_value . "_" . $tolerance_type . "bin.txt";
	
	open (MASSLIST,$filename)||die "Could not open mass list file: $filename";
	while (<MASSLIST>){
		chomp;
		$line = $_;
		#print "$line\n";
		push (@mass_array, $line);
	}
	close (MASSLIST) || die "Couldn't close mass list file!\n";
	@mass_array = sort {$a <=> $b}(@mass_array);
	$lower_mass = 218.14;
	#$tolerance_type = "ppm";
	#print "peptide filename is $filename\n";
	#print "tolerance type $tolerance_type\n";
	#print "tolerance value $tolerance_value\n";
	#print "start bin $lower_mass\n";
	#print "end bin $endmass\n";
	#print "number of masses to bin $#mass_array\n";
	#print "@mass_array\n";
	#sleep 3;
	
	open (OUTBIN,">>$newfilename")||die "Could not open output file: $newfilename";
	while ($lower_mass <$endmass){
		$mass_window = cal_mass_tolerance ($lower_mass, $tolerance_type, $tolerance_value );
		$upper_mass = $lower_mass + $mass_window;
		$graph_value = ($lower_mass + $upper_mass) /2;
		$bin =0;
		#print "mass win is $mass_window upper $upper_mass lower $lower_mass graph $graph_value\n";
		for  ($i = 1; $i <= $#mass_array; $i++){
			$mass_out = shift (@mass_array);
			#print  "$mass_out\n";
			if ($mass_out < $lower_mass){
				next;
			}
			if ($mass_out > $upper_mass){
				unshift (@mass_array, $mass_out);
				last;
			}
			if ($mass_out >= $lower_mass){
				if ($mass_out < $upper_mass){ 
					$bin ++;
					#print "We got a hit mass $mass_out lower $lower_mass upper $upper_mass bin $bin\n";
				}
			}
		}
		unless ($bin == 0){
			$unique_percent = (1/$bin) * 100;
			print OUTBIN "$graph_value\t$bin\t$unique_percent\n";
			#print "graph value $graph_value count $bin unique $unique_percent\%\n";
		}
		$lower_mass = $upper_mass;
	}
	close (OUTBIN) || die "Couldn't close output file file!\n";	
}
	
#######################
#cal_mass_tolerance Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub cal_mass_tolerance {
	my ($mass, $tolerance_type, $tolerance_value) = @_;
	my $mass_tolarance = 0;
	if ($tolerance_type eq "ppm"){
		#calculate mass tolerance in ppm
		$mass_tolarance = (($tolerance_value/1e6) * $mass);
	}
	#if tolerance eq Da	
	elsif ($tolerance_type  eq "Da"){
		#use tolerance value 
		$mass_tolarance = $tolerance_value;
	}
	#if tolerance eq mmu	
	elsif ($tolerance_type  eq "mmu"){
		#use tolerance value 
		$mass_tolarance = ($tolerance_value /1000);
	}
	return $mass_tolarance;
}

#######################
#aa_analysis Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub aa_analysis {
	my ($Output_filename_list_ref) = @_;
	#my @AAlist = &aa_list;
	my ($AA,
		$file,
		@StatFiles
	);
	@AAlist = sort (@AAlist);
	foreach $AA (@AAlist){
		print "Analysing $AA list output\n";
		$file = $Output_filename_list_ref->{$AA};
		open (IN, $file) or die "can not open file for reading: $!";
		#output
		#No of AA residudes in Protein hash
		#key = No of AA
		#Value = No of Proteins
		my %AA_proteins = ();
		#No of AA containing peptides per a protein
		#key = No of AA
		#value = No of Peptides
		my %AA_peptides = ();
		#No of AA per a peptide
		#key = No of AA
		#value = No of Peptides
		my %AA_in_peptide = ();
		my $TotalNoOfProteins = 0;
		my $TotalNoOfPeptides = 0;
		my $TotalNoAAProteins =0;
		my $TotalAAPep = 0;
		my $NoOfAAinDatabase = 0;
		my $NoOfAllAAinDatabase = 0;
		#generate output filename from input file
		my @file_split =  split (/\_/,$file);
		my $outfile = $file_split[0] . "_" . $file_split[1] . "_" . $file_split[2] ."_stat.txt";
		push (@StatFiles, $outfile);
		#Main
		#for each line
		while (<IN>){
			chomp;
			my $line = $_;
			if (/^\d/){
				#input data format
				#1		2		3	4	5		6		7		8		9		10				
				#MS-Digest No. \t Accession No.\t MW \t PI \t No of AA \t Total No of AA \t % AA \t No of AA peptides \t No of Peptides \t % of AA containing peptides 
				#10368	P08131	18810.8	6.03	41	182	22.52%	3	3	100%	935.3980	2	WCRPDCR	6753.5982	18	VEGTSLPPCCVVSCTSPSCCQLYYAQASCCRPSYCGQSCCRPACCCQPTCIEPVCEPTCCEPTC	10917.3718	21	MACCSTSFCGFPTCSTGGTCGSNFCQPTCCQTSCCQPTSIQTSCCQPTSIQTSCCQPTSIQTSCCQPISIQTSCCQPTCLQTSGCETGCGIGGSIGYGQVGSSGAVSSR
				#\d+\.\d+\t(\d+)\t\w+
				#peptide info: Mass	No. AA	RetentionTime	Sequence
				#671.3299	1	18.4	NGHICK
				#split line in to list by tab
				my @line_split =  split (/\t/,$line);
				$TotalNoOfProteins ++;
				$TotalNoOfPeptides = $TotalNoOfPeptides + $line_split[8];
				$NoOfAllAAinDatabase = $NoOfAllAAinDatabase + $line_split[5];
				$NoOfAAinDatabase = $NoOfAAinDatabase + $line_split[4];
				if ($line_split[7] == 0 ){
					next;
				}
				else {
					$TotalNoAAProteins ++;
				}
				#read in No of AA residudes in Protein
				#$linesplit[4]
				my $Protein_AA = $line_split[4];
				#if number aready exists then add 1 to number of times it has occured
				if($AA_proteins{$Protein_AA}){
					#increase no of AA in protein value by 1
					$AA_proteins{$Protein_AA} ++;
				}
				else {
					#place number in to a hash.
					$AA_proteins{$Protein_AA} = 1;
				}
				#if protein does not contain any AA then go to next line.
				#read in number of AA contain peptides
				#$linesplit[7] = No of AA containing peptides
				my $Peptide_AA = $line_split[7];
				$TotalAAPep = $TotalAAPep + $Peptide_AA;
				if ($Peptide_AA == 0){
					next;
				}
				#if $no_of_AA_containing peptides >=1
				if ($AA_peptides{$Peptide_AA}){
					#increase no of AA peptides value by 1
					$AA_peptides{$Peptide_AA} ++;
				}
				else {
					#place number in to a hash.
					$AA_peptides{$Peptide_AA} = 1;
				}
				#for each peptide
				#set linesplit position
				#from position 11 add 4 for the next till END
				my $linesplit_pos = 11;
				for (my $j = 1; $j <= $Peptide_AA; $j++){
					#grab no of AA
					my $AA_in_pep = $line_split[$linesplit_pos];
					#add key to hash if already exists then value ++
					if ($AA_in_pep =~ /^\D/){
						next;
					}
					if($AA_in_peptide{$AA_in_pep}){
						#increase no of AA in protein value by 1
						$AA_in_peptide{$AA_in_pep} ++;
					}
					else {
						#place number in to a hash.
						$AA_in_peptide{$AA_in_pep} = 1;
					}
					#Add 3 to index No
					$linesplit_pos = ($linesplit_pos + 4);
					#end for each
				}
				#end for AA containing peptides
			}
			#end for each line
		}
		#close output file
		close (IN) || die "Couldn't close input file!\n";
		#sort hash proteins
		my @AA_protein_list = keys(%AA_proteins);
		@AA_protein_list = sort {$a <=> $b}(@AA_protein_list);
		#sort hash peptides
		my @AA_peptide_list = keys(%AA_peptides);
		@AA_peptide_list = sort {$a <=> $b}(@AA_peptide_list);
		#sort hash AA in peptides
		my @AA_in_peptide_list = keys(%AA_in_peptide);
		@AA_in_peptide_list = sort {$a <=> $b}(@AA_in_peptide_list);
		
		my $total_AA_ratio = ($NoOfAAinDatabase/$NoOfAllAAinDatabase)*100;
		$total_AA_ratio = number_split ($total_AA_ratio, 2);
		my $total_AA_peptide_ratio = ($TotalAAPep/$TotalNoOfPeptides)*100;
		$total_AA_peptide_ratio = number_split ($total_AA_peptide_ratio, 2);
		my $total_AA_protein_ratio = (($TotalNoAAProteins)/$TotalNoOfProteins)*100;
		$total_AA_protein_ratio = number_split ($total_AA_protein_ratio, 2);
		
		#open output file
		open (OUT, ">$outfile")|| die "can't open output file";

		#print stats for AA		
		print OUT "Total number of Amino Acids =\t$NoOfAllAAinDatabase\n";
		print OUT "Total number of $AA Amino Acid =\t$NoOfAAinDatabase\n";
		print OUT "$AA ratio = $total_AA_ratio%\n\n";
		print OUT "Total number of  peptides =\t$TotalNoOfPeptides\n";
		print OUT "Total number of $AA containing peptides =\t$TotalAAPep\n";
		print OUT "$AA peptide ratio = $total_AA_peptide_ratio%\n\n";
		print OUT "Total number of proteins \t$TotalNoOfProteins\n";		
		print OUT "Total number of proteins containing $AA\t$TotalNoAAProteins\n";
		print OUT "$AA protein ratio = $total_AA_protein_ratio%\n\n";
		
		#print table header for proteins
		print OUT "Number of $AA per a Protein\n";
		print OUT "Number of $AA\tNumber of Proteins\n";
		#print each value from hash keys
		foreach my $keyvalue (@AA_protein_list){
			print OUT "$keyvalue\t$AA_proteins{$keyvalue}\n";
		}
		#print table header for peptides
		print OUT "\nNumber of $AA containing peptides per a Protein\n";
		print OUT "Number of $AA peptides\tNumber of Proteins\n";
		#print each value from hash keys
		foreach my $keyvalue (@AA_peptide_list){
			print OUT "$keyvalue\t$AA_peptides{$keyvalue}\n";
		}
		#print table header for peptides
		print OUT "\nNumber of $AA per a Peptide\n";
		print OUT "Number of $AA\tNumber of Peptides\n";
		#print each value from hash keys
		foreach my $keyvalue (@AA_in_peptide_list){
			print OUT "$keyvalue\t$AA_in_peptide{$keyvalue}\n";
		} 
		#close output file
		close (OUT) || die "Couldn't close output file!\n";
	}
	return (@StatFiles);
}
#######################
#aa_list Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub aa_list {
	my @AAlist = ("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y");
	return (@AAlist);
}
#######################
#aa_list Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub build_toleranceValue_list {
	my @toleranceValueAMT = qw(0.1 0.5 1 5 10 15 20 25 30 40 50 75 100 200 250 300 400 500 750 1000);
	return (@toleranceValueAMT);
}

#######################
#cal_retention_time Subroutine
#Richard Jacob
#Copyright (c) 2004 KBiosystems
#
#Description
#calculate retention time according to the paper by Krokhin OV et al. :
#An improved model for prediction of retention times of tryptic peptides in ion
#pair reversed-phase HPLC: its application to protein peptide mapping by
#off-line HPLC-MALDI MS.
#Krokhin OV, Craig R, Spicer V, Ens W, Standing KG, Beavis RC, Wilkins JA
#Mol Cell Proteomics. 2004 Sep;3(9):908-19. Epub 2004 Jul 06
#######################
sub cal_retention_time {
	my ($Sequence, $a , $b) = @_;
	my @AAlist = &aa_list;
	my (
		$retentionTime,
		%AARcHash,
		%AAcNtHash,
		$seqLenght,
		$Kl,
		$Hydrophobicty,
		$sumRc,
		$cNt,
		$length,
		$R1,
		$R2,
		$R3,
		$R1cNt,
		$R2cNt,
		$R3cNt,
		$AARctotal,
		$NoofAA,
		$AARc,
	);
	#Generate retention coefficient hashes
	%AARcHash = ("A", "0.8", "R", "-1.3", "N", "-1.2", "D", "-0.5", "C", "-0.8", "E", "0.0", "Q", "-0.9", "G", "-0.9", "H", "-1.3", "I", "8.4", "L", "9.6", "K", "-1.9", "M", "5.8", "F", "10.5", "P", "0.2", "S", "-0.8", "T", "0.4", "W", "11", "Y", "4.0", "V", "5.0", "X", "0");
	%AAcNtHash = ("A", "-1.5", "R", "8.0", "N", "5.0", "D", "9.0", "C", "4.0", "E", "7.0", "Q", "1.0", "G", "5.0", "H", "4.0", "I", "-8.0", "L", "-9.0", "K", "4.6", "M", "-5.5", "F", "-7.0", "P", "4.0", "S", "5.0", "T", "5.0", "W", "-4.0", "Y", "-3.0", "V", "-5.5", "X", "0");
	
	#calculate Kl value
	$length = length($Sequence);
	if ($length <10){
		$Kl = (1-0.027)*(10-$length);
	}
	elsif ($length>20){
		$Kl = (1-0.014)*($length-20);
	}
	else {
		$Kl = 1;
	}
		
	#calculate sum of Rc
	my %AAcounthash = &countAA ($Sequence);
	$sumRc =0;
	foreach my $key (keys (%AAcounthash)){
		$NoofAA = 0;
		$NoofAA = $AAcounthash{$key};
		$AARc = $AARcHash{$key};
		if ($NoofAA >=1){
			$AARctotal = $NoofAA * $AARc;
		}
		else {
			$AARctotal = 0;
		}
		$sumRc = ($sumRc + $AARctotal);
	}
	
	#calculate Rx cNt values 
	$R1 = substr $Sequence, 0, 1;
	$R1cNt = 0.42*$AAcNtHash{$R1};
	if ($length > 1){
		$R2 = substr $Sequence, 1, 1;
		$R2cNt = 0.22*$AAcNtHash{$R2};
		if ($length >2){
			$R3 = substr $Sequence, 2, 1;
			$R3cNt = 0.05*$AAcNtHash{$R3};
		}
		else {
			$R3cNt= 0;
		}
	}
	else {
		$R2cNt= 0;
		$R3cNt= 0;
	}
	
	$Hydrophobicty = $Kl *($sumRc + $R1cNt + $R2cNt + $R3cNt);
	if ($Hydrophobicty >= 38){
		$Hydrophobicty = $Hydrophobicty - (0.3 * ($Hydrophobicty -38));
	}
	$retentionTime = $a + ($b*$Hydrophobicty);
	return ($retentionTime);
}
#######################
#cal_mono_isotopic_mass Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub cal_mono_isotopic_mass {
	my ($Sequence) = @_;
	my @AAlist = &aa_list;
	my $monomass = 0;
	#Count the number of each amino acid in sequence and put it in to a aa key number value hash
	my %AAcounthash = &countAA ($Sequence);
	my %MassMonoIsotopicHash=("A", "71.03712", "R", "156.10112", "N", "114.04293", "D", "115.02695", "C", "103.00919", "E", "129.04260", "Q", "128.05858", "G", "57.02147", "H", "137.05891", "I", "113.08407", "L", "113.08407", "K", "128.09497", "M", "131.04049", "F", "147.06842", "P", "97.05277", "S", "87.03203", "T", "101.04768", "W", "186.07932", "Y", "163.06333", "V", "99.06842", "X", "100");
	foreach my $key (keys (%AAcounthash)){
		my $NoofAA = $AAcounthash{$key};
		my $AAmass = $MassMonoIsotopicHash{$key};
		my $AAmasstotal = $NoofAA * $AAmass;
		$monomass = ($monomass + $AAmasstotal);
	} 
	#add c and n terminal 
	my $cterm = 17.002739654;
	my $nterm = 1.007825032;
	my $hydrogen = 1.007825032;
	$monomass = ($monomass + $cterm + $nterm);
	#add +1 hydrogen charge
	$monomass = ($monomass + $hydrogen);
	#print "Mono Isotopic mass is $monomass\n";
	return $monomass;
}

#######################
#cal_average_mass Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub cal_average_mass {
	my ($Sequence) = @_;
	my @AAlist = &aa_list;
	my $monomass = 0;
	#Count the number of each amino acid in sequence and put it in to a aa key number value hash
	my %AAcounthash = &countAA ($Sequence);
	my %MassMonoIsotopicHash=("A", "71.08", "R", "156.19", "N", "114.10", "D", "115.09", "C", "103.14", "E", "129.12", "Q", "128.13", "G", "57.05", "H", "137.14", "I", "113.16", "L", "113.16", "K", "128.17", "M", "131.19", "F", "147.18", "P", "97.12", "S", "87.08", "T", "101.10", "W", "186.21", "Y", "163.18", "V", "99.13", "X", "100");
	foreach my $key (keys (%AAcounthash)){
		my $NoofAA = $AAcounthash{$key};
		my $AAmass = $MassMonoIsotopicHash{$key};
		my $AAmasstotal = $NoofAA * $AAmass;
		$monomass = ($monomass + $AAmasstotal);
	} 
	#add c and n terminal 
	my $cterm = 17.0027;
	my $nterm = 1.0078;
	my $hydrogen = 1.0078;
	$monomass = ($monomass + $cterm + $nterm);
	#add +1 hydrogen charge
	$monomass = ($monomass + $hydrogen);
	#print "Mono Isotopic mass is $monomass\n";
	return $monomass;
}

#######################
#cal_pI Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub cal_pI {
	my ($seq) = @_;
	my %counts = &countAA ($seq);
	#my %counts = ("A"=>0, "R"=>0, "N"=>0, "D"=>0, "C"=>0,
				#"Q"=>0, "E"=>0, "G"=>0, "H"=>0, "I"=>0,
				#"L"=>0, "K"=>0, "M"=>0, "F"=>0, "P"=>0,
				#"S"=>0, "T"=>0, "W"=>0, "Y"=>0, "V"=>0
				#); 
	## Hash of pKa values used in the calculation-- 
	#  pKa[0] = 'internal residue pKa'
	#  pKa[1] = 'C-term residue pKa'
	#  pKa[2] = 'N-term residue pKa'
	my %pKa = ("A"=>[0, 3.55, 7.59],
			   "R"=>[12.0, 3.55, 7.50],
			   "N"=>[0, 3.55, 7.50],
			   "D"=>[4.05, 4.55, 7.50],
			   "C"=>[9.00, 3.55, 7.50],
			   "Q"=>[0, 3.55, 7.50],
			   "E"=>[4.45, 4.75, 7.70],
			   "G"=>[0, 3.55, 7.50],
			   "H"=>[5.98, 3.55, 7.50],
			   "I"=>[0, 3.55, 7.50],
			   "L"=>[0, 3.55, 7.50],
			   "K"=>[10.00, 3.55, 7.50],
			   "M"=>[0, 3.55, 7.00],
			   "F"=>[0, 3.55, 7.50],
			   "P"=>[0, 3.55, 8.36],
			   "S"=>[0, 3.55, 6.93],
			   "T"=>[0, 3.55, 6.82],
			   "W"=>[0, 3.55, 7.50],
			   "Y"=>[10.00, 3.55, 7.50], 
			   "V"=>[0, 3.55, 7.44]
			  );

	#for (my $x = 1; $x < (length($seq) - 1); $x++) {
	#	 my $key = substr $seq, $x, 1;
	#	 $counts{$key} += 1;
	# }
	 
	my $length = length($seq);
	my $Nterm = substr $seq, 0, 1;
	my $Cterm = substr $seq, -1, 1;
	
	my $Dnum = $counts{"D"};
	my $Rnum = $counts{"R"};
	my $Hnum = $counts{"H"};
	my $Knum = $counts{"K"};
	my $Enum = $counts{"E"};
	my $Cnum = $counts{"C"};
	my $Ynum = $counts{"Y"};
	
	my $pHmin = 0.00;
	my $pHmax = 14.00;
	
	my $pH = 7.0;
	my $negative = my $positive = 0.0;
	for (my $i = 0; $i < 1000; $i++) {
		$pH = ($pHmin + ($pHmax - $pHmin)/2.0);
		$negative = (
			-$Dnum*(1/(1 + 10**(-$pH + $pKa{"D"}[0])))
			- $Enum*(1/(1 + 10**(-$pH + $pKa{"E"}[0])))
			- $Cnum*(1/(1 + 10**(-$pH + $pKa{"C"}[0])))
			- $Ynum*(1/(1 + 10**(-$pH + $pKa{"Y"}[0])))
			- (1/(1 + 10**(-$pH + $pKa{$Cterm}[1])))
		); 
		$positive = (
			$Hnum*(1/(1 + 10**(-$pKa{"H"}[0] + $pH)))
            + $Knum*(1/(1 + 10**(-$pKa{"K"}[0] + $pH)))
            + $Rnum*(1/(1 + 10**(-$pKa{"R"}[0] + $pH)))
            + (1/(1 + 10**(-$pKa{$Nterm}[2] + $pH)))
		);
		my $total_charge = $negative + $positive;
		if (abs($total_charge) < 0.01) {
			return $pH;
		}
		if ($total_charge > 0.0) {
			$pHmin = $pH;
		} else {
			$pHmax = $pH;
		}
	}
	# shouldn't get here, but if it does...
	print "Didn't converge in 1000 iterations\n"; 
}


#######################
#countAA Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub countAA {
	my ($Sequence) = @_;
	my %AAcounthash = ();
	my @AAlist = &aa_list;
	foreach my $AA (@AAlist){
		my $AAcount = 0;
		if ($Sequence=~ /$AA/){
			$AAcount ++ while $Sequence=~ /$AA/g;
			#print "for $AA the count is $AAcount";#bugcheck
			$AAcounthash{$AA}=$AAcount;
		}
		else {
			$AAcounthash{$AA}=0;
		}
	}
	return %AAcounthash;
}
#######################
#digestProtein Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub digestProtein {
	my ($protein, $enzymeName) = @_;
	#enzyme parameters set once as global variables to save time	
	#digest protein sequence
	#step through sequence forward for c and n terminal cleavges
	my $proteinLenght = length $protein;
	my ($currentAA,
		$aa_current,
		$exception,
		$peptide,
		$cut,
		$cutAA,
		%cleavgePeptides,
		$AAstart,
		$AAend,
		@peptideinfo,
		$monomass,
		$averagemass,
		$i,
		$nextAA,
		$previousAA,
		$retentionTime,
		);
	$cut = 0;
	$AAstart = 1;
	$nextAA = "X";
	$previousAA = "X";
	$i = 1;
	for ($currentAA = 0; $currentAA < $proteinLenght; $currentAA++){
		$aa_current = substr($protein, $currentAA, 1);
		#set aa+1 to aa
		if ($currentAA == $proteinLenght){
			#if its the end of the protein set next aa to x
			$nextAA = "X";
			$previousAA = substr($protein, ($currentAA-1), 1);
		}
		elsif ($currentAA == 0){
			#if its the start of the protein set previous aa to x
			$nextAA = substr($protein, ($currentAA+1), 1);
			$previousAA = "X";
		}
		else{
			#else set next and previous aa
			$nextAA = substr($protein, ($currentAA+1), 1);
			$previousAA = substr($protein, ($currentAA-1), 1);
		}
		#if aa is a cleaveAA
		foreach $cutAA (@cleaveCTerm_array){
			if ($aa_current eq $cutAA){
				#and aa-1 is not a cleave exception
				$cut = "C";
				foreach $exception (@cleaveCTermException_array){
					if ($nextAA eq $exception){
						$cut = "X";
						last;
					}
				}
			}
		}
		foreach $cutAA (@cleaveNTerm_array){
			if ($aa_current eq $cutAA){
				#and aa-1 is not a cleave exception
				$cut = "N";
				foreach $exception (@cleaveCTermException_array){
					if ($previousAA eq $exception){
						$cut = "X";
						#copy aa to peptide temp
						$peptide = $peptide . $aa_current;
						last;
					}
				}
			}
		}
		
		if (($currentAA+ 1) == $proteinLenght){
			$cut = "C";
		}	
		if ($cut eq "C"){
			#copy aa to peptide temp
			$peptide = $peptide . $aa_current;
			#move temp peptide to cleavge product peptide
			$AAend = $currentAA +1;
			$monomass = &cal_mono_isotopic_mass ($peptide);
			$averagemass = &cal_average_mass ($peptide);
			$retentionTime = &cal_retention_time ($peptide, $Hydro_a, $Hydro_b);
			@peptideinfo =( $peptide, $AAstart, $AAend, $monomass, $averagemass, $retentionTime);
			$cleavgePeptides{$i}=[@peptideinfo];
			#re-initalize temp peptide
			#print "$i $peptide\t$AAstart\t$AAend\t$monomass\t$averagemass\t$retentionTime\n";
			$peptide = "";
			$cut =0;
			$AAstart = $currentAA +2;
			$i++;
			next;
		}
		elsif ($cut eq "N"){
			#copy aa to peptide temp
			#move temp peptide to cleavge product peptide
			$AAend = $currentAA;
			$monomass = &cal_mono_isotopic_mass ($peptide);
			$averagemass = &cal_average_mass ($peptide);
			$retentionTime = &cal_retention_time ($peptide,$Hydro_a, $Hydro_b );
			@peptideinfo =( $peptide, $AAstart, $AAend, $monomass, $averagemass, $retentionTime);
			$cleavgePeptides{$i}=[@peptideinfo];
			#re-initalize temp peptide
			#print "$i $peptide\t$AAstart\t$AAend\t$monomass\t$averagemass\t$retentionTime\n";
			$peptide = $aa_current;
			$cut ="X";
			$AAstart = $currentAA+1;
			$i++;
			next;
		}
		else{
			#next aa
			$peptide = $peptide . $aa_current;
		}
	}
	return 	(\%cleavgePeptides);
}

#######################
#processProtein Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub processProtein {
	my ($protein, $index_no, $enzymeName, $AcessionNo, $NoOfAllAAinDatabase, $TotalNoOfPeptides, $Output_filename_proteins_hash_ref, $Output_filename_peptides_hash_ref, $Output_filename_hash_ref, $AATotalhash_ref) = @_;
	#initilasie variables
	#counters
	my $i = 0;
	my $NoOfPeptides = 0;
	my $NoOfAAPeptides = 0;
	my %AA_peptides = ();
	my %AA_Counts = ();
	#pI
	my ($pI) = &cal_pI($protein);
	#my $MS_Digest_No = $index_no;
	my ($proteinMW) = &cal_mono_isotopic_mass ($protein);
	#No of AA
	%AA_Counts = &countAA ($protein);
	foreach my $key (keys (%AA_Counts)){
		if ($AA_Counts{$key}>=1){
			if ($AATotalhash_ref->{$key}){
				$AATotalhash_ref->{$key} = $AATotalhash_ref->{$key} + $AA_Counts{$key};
			}
			else {
				$AATotalhash_ref->{$key} = $AA_Counts{$key};
			}
		}
		
	}
	#digest protein
	my $peptidearrayReference = digestProtein ($protein, $enzymeName);
	%AA_peptides = %{$peptidearrayReference};
	my @tempKeys = (keys (%AA_peptides));
	$NoOfPeptides = ( $#tempKeys +1);
	$TotalNoOfPeptides = $TotalNoOfPeptides + $NoOfPeptides;
	my $Total_NoOfAllAA = length ($protein);
	$NoOfAllAAinDatabase = ($NoOfAllAAinDatabase + $Total_NoOfAllAA);
	#print output for all peptides
	print OUTPUT "$index_no\t$AcessionNo\t$proteinMW\t$pI\t$Total_NoOfAllAA\t$NoOfPeptides";
	#foreach pep in hash
	print OUTPUTPRO "$proteinMW\n";
	foreach my $pep (@tempKeys){
		print OUTPUT "\t$AA_peptides{$pep}[4]\t$AA_peptides{$pep}[3]\t$AA_peptides{$pep}[5]\t$AA_peptides{$pep}[0]";
		print OUTPUTPEP "$AA_peptides{$pep}[3]\n";
	}
	#close line print \n
	print OUTPUT "\n";
	
	my $fh_pro;
	#print output for individual peptides
	foreach my $AA (@AAlist){
		if ($AA_Counts{$AA}){
			$fh_pro = $Output_filename_proteins_hash_ref->{$AA};
			print $fh_pro "$proteinMW\n";
		}
	}
	
	
	my ( $percentAA, $tempAAPepCount, $percentPep, $AAcount);
	foreach my $AA (@AAlist){
		$tempAAPepCount = 0;
		my $fh = $Output_filename_hash_ref->{$AA};
		my $fh_pep = $Output_filename_peptides_hash_ref->{$AA};
		if ($AA_Counts{$AA}){
			$percentAA = ($AA_Counts{$AA}/$Total_NoOfAllAA) *100;
			$percentAA = number_split ($percentAA, 2);
		}
		else {
			$percentAA = 0;
		}					
		foreach my $pep (@tempKeys){
			if ($AA_peptides{$pep}[0]=~ /$AA/){
				$tempAAPepCount ++;
			}
		}
		if ($tempAAPepCount >= 1){
			$percentPep= ($tempAAPepCount/$NoOfPeptides)*100;
			$percentPep = number_split ($percentPep, 2);
		}
		else{
			$percentPep=0;
		}
		print $fh "$index_no\t$AcessionNo\t$proteinMW\t$pI\t$AA_Counts{$AA}\t$Total_NoOfAllAA\t$percentAA%\t$tempAAPepCount\t$NoOfPeptides\t$percentPep%";
		
		foreach my $pep (@tempKeys){
			if ($AA_peptides{$pep}[0]=~ /$AA/){
				$AAcount = 0;
				$AAcount ++ while $AA_peptides{$pep}[0]=~ /$AA/g;
				print $fh  "\t$AA_peptides{$pep}[3]\t$AAcount\t$AA_peptides{$pep}[5]\t$AA_peptides{$pep}[0]";
				print $fh_pep  "$AA_peptides{$pep}[3]\n";
			}
		}
		print $fh "\n";
	}
	return ($NoOfAllAAinDatabase, $TotalNoOfPeptides, $AATotalhash_ref);
}
#######################
#generateStats Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Generates summaray stats from the individual stats  files 
#
#######################
sub generateStats {
	my (@files_to_process) = @_;
	my ($filename,
		@filename_split,
		$databaseName,
		$enzymeName,
		$currentAA,
		%AAStats,
		$AA,
		@ResultsTextArray,
		$ResultsCounter
		);
	#for all files in current directory	
	foreach $filename (@files_to_process) {
		#print "current file $filename\n";
		#split file name to determine genome and aa
		@filename_split = split (/_/,$filename);
		#Input
		#database name
		$databaseName = $filename_split[0];
		$enzymeName = $filename_split[1];
		$currentAA = $filename_split[2];
		#open file
		#foreach line
		my ($line,
			$Total_All_AA,
			$total_no_current_AA,
			$AA_ratio,
			$total_peptides,
			$total_AA_peptides,
			$Pep_ratio,
			$No_proteins,
			$total_AA_proteins,
			$protein_ratio,
			@temp_filestats,
			$InAAProtein,
			$InAAPeptide,
			$InAAConPeptide,
			$InAAProtein_counter,
			$InAAPeptide_counter,
			$InAAConPeptide_counter,
			@linesplit,
		);        
		$InAAProtein =0;
		$InAAPeptide = 0;
		$InAAConPeptide = 0;
	
		open (INPUT, "$filename")|| die "cant open input file\n";
		while (<INPUT>){
			$line = $_;
			chomp $line;
			#put key values in to a list - then a hash
			if ($line =~ /Total number of Amino Acids/){
				#Total number of Amino Acids =	2974038
				($Total_All_AA) = ($line =~ /(\d+)/);
				push (@temp_filestats, $Total_All_AA);
				next;
			}
			if ($line =~ /Total number of $currentAA Amino Acid \=	/){
				#Total number of $currentAA Amino Acid =	38775
				($total_no_current_AA) = ($line =~ /(\d+)/);
				push (@temp_filestats, $total_no_current_AA);
				next;
			}
			if ($line =~ /$currentAA ratio/){
				#AA ratio = 1.30%
				($AA_ratio) = ($line =~ /([0-9.]+)/);
				push (@temp_filestats, $AA_ratio);
				next;
			}
			if ($line =~ /Total number of  peptides/){
				#Total number of  peptides =	341702
				($total_peptides) = ($line =~ /(\d+)/);
				push (@temp_filestats, $total_peptides);
				next;
			}
			if ($line =~ /Total number of $currentAA containing peptides/){
				#Total number of AA containing peptides =	31601
				($total_AA_peptides) = ($line =~ /(\d+)/);
				push (@temp_filestats, $total_AA_peptides);
				next;
			}
			if ($line =~ /$currentAA peptide ratio/){
				#AA peptide ratio = 9.24%
				($Pep_ratio) = ($line =~ /([0-9.]+)/);
				push (@temp_filestats, $Pep_ratio);
				next;
			}
			if ($line =~ /Total number of proteins/){
				#Total number of proteins
				($No_proteins) = ($line =~ /(\d+)/);
				push (@temp_filestats, $No_proteins);
				next;
			}
			if ($line =~ /Total number of proteins containing $currentAA/){
				#Total number of proteins containing AA	5768
				($total_AA_proteins) = ($line =~ /(\d+)/);
				push (@temp_filestats, $total_AA_proteins);
				next;
			}
			if ($line =~ /$currentAA protein ratio/){
				#AA protein ratio = 91.58%
				($protein_ratio) = ($line =~ /([0-9.]+)/);
				push (@temp_filestats, $protein_ratio);
				next;
			}
			if ($line =~ /Number of $currentAA per a Protein/){
				#Number of $currentAA per a Protein
				#print "current AA in protein\n";
				$InAAProtein = 1;
				$InAAProtein_counter = 1;
				next;
			}
			if ($InAAProtein ==1){
				$InAAProtein_counter ++;
				if ($InAAProtein_counter > 2){
					if ($line =~ /^\d/){
							#print "$line\n";
							@linesplit = split (/\t/, $line);
							$InAAProtein_hash{$currentAA}{$linesplit[0]} = $linesplit[1];
						}
					else {
							$InAAProtein = 0;
							$InAAProtein_hash{$currentAA}{'highest'}= $linesplit[0];
							next;
					}
				}
			}
			if ($line =~ /Number of $currentAA containing peptides per a Protein/){
				#Number of $currentAA per a Protein
				#print "current AA in con peptide\n";
				$InAAConPeptide= 1;
				$InAAConPeptide_counter = 1;
				next;
			}
			if ($InAAConPeptide ==1){
				$InAAConPeptide_counter ++;
				if ($InAAConPeptide_counter > 2){
					if ($line =~ /^\d/){
							#print "$line\n";
							@linesplit = split (/\t/, $line);
							$InAAConPeptide_hash{$currentAA}{$linesplit[0]} = $linesplit[1];
						}
					else {
							$InAAConPeptide = 0;
							$InAAConPeptide_hash{$currentAA}{'highest'}= $linesplit[0];
							next;
					}
				}
			}
			 if ($line =~ /Number of $currentAA per a Peptide/){
				#Number of $currentAA per a Protein
				#print "current AA in con peptide\n";
				$InAAPeptide= 1;
				$InAAPeptide_counter = 1;
				next;
			}
			if ($InAAPeptide ==1){
				$InAAPeptide_counter ++;
				if ($InAAPeptide_counter > 2){
					if ($line =~ /^\d/){
						#print "$line\n";
						@linesplit = split (/\t/, $line);
						$InAAPeptide_hash{$currentAA}{$linesplit[0]} = $linesplit[1];
					}
				}
			}
		}
		$InAAPeptide_hash{$currentAA}{'highest'}= $linesplit[0];
		$AAStats{$currentAA}= [@temp_filestats];
		#close file
		close INPUT|| die "Couldn't close input file!\n";
	}
	#print Dumper \%AAStats;
	#print Dumper (%InAAProtein_hash);
	#print Dumper (%InAAConPeptide_hash);
	#print Dumper (%InAAPeptide_hash);
	#
	
	#output collated resutls in to a tab delimited table
	#generate output filename
	#open output file
	my $Output_filename = $databaseName . "_" . $enzymeName ."_results.txt";
	open( RESULTS, ">$Output_filename" ) or die( "Bad things happened : $!" );
	#Build an AA list
	foreach $AA (@AAlist){
		print RESULTS "\t$AA";
	}
	@ResultsTextArray = ("\nTotal number of all AA","\nTotal number of individual AA","\nAA Ratio","\nTotal number of all peptides","\nTotal number of AA peptides","\nTotal number of AA peptides ratio","\nTotal number of Proteins","\nTotal number of AA Proteins","\nTotal number of AA Proteins ratio");
	for ($ResultsCounter = 0; $ResultsCounter <= 8; $ResultsCounter ++){
		print RESULTS "$ResultsTextArray[$ResultsCounter]";
		foreach $AA (@AAlist){
			print RESULTS "\t$AAStats{$AA}[$ResultsCounter]";
		}
	}

	print RESULTS "\n\nNumber of AA per a Protein";
	foreach $AA (@AAlist){
		print RESULTS "\t$AA";
	}
	#work out the highest value
	my @highest = sort by_highest_AAProtein keys(%InAAProtein_hash);
	#print "highest AA is $highest[$#highest] with a value of ";
	#print "$InAAProtein_hash{$highest[$#highest]}{'highest'}\n";
	#for each value type in hash
	for (my $i = 1; $i<= $InAAProtein_hash{$highest[$#highest]}{'highest'}; $i++){
		print RESULTS "\n$i";
		#for each AA
		foreach $AA (@AAlist){
			if ($InAAProtein_hash{$AA}{$i}){
				#print out results
				print RESULTS "\t$InAAProtein_hash{$AA}{$i}";
			}
			else {
				print RESULTS "\t0";
			}
		}
	}
	
	#print out results
	#may also need further processing for some 
	#eg no. of proteins with more than 3 AA containing peptides
	#
	@highest = ();
	@highest = sort by_highest_AAConPeptide keys(%InAAConPeptide_hash);
	print RESULTS "\n\nNumber of AA containing peptides per a Protein";
	foreach $AA (@AAlist){
		print RESULTS "\t$AA";
	}
	for (my $i = 1; $i<= $InAAConPeptide_hash{$highest[$#highest]}{'highest'}; $i++){
		print RESULTS "\n$i";
		#for each AA
		foreach $AA (@AAlist){
			if ($InAAConPeptide_hash{$AA}{$i}){
				#print out results
				print RESULTS "\t$InAAConPeptide_hash{$AA}{$i}";
			}
			else {
				print RESULTS "\t0";
			}
		}
	}
	
	@highest = ();
	@highest = sort by_highest_AAPeptide keys(%InAAPeptide_hash);
	print RESULTS "\n\nNumber of AA peptides per a Protein";
	foreach $AA (@AAlist){
		print RESULTS "\t$AA";
	}
	for (my $i = 1; $i<= $InAAPeptide_hash{$highest[$#highest]}{'highest'}; $i++){
		print RESULTS "\n$i";
		#for each AA
		foreach $AA (@AAlist){
			if ($InAAPeptide_hash{$AA}{$i}){
				#print out results
				print RESULTS "\t$InAAPeptide_hash{$AA}{$i}";
			}
			else {
				print RESULTS "\t0";
			}
		}
	}
	#close output file
	close RESULTS|| die "Couldn't close file $Output_filename\n";
	return ();
}
#######################
#by_highest_AAProtein Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub by_highest_AAProtein {
    #print "a $a b $b\n";
    #print "a $InAAProtein_hash{$a}{'highest'} b is $InAAProtein_hash{$b}{'highest'}\n";
    return $InAAProtein_hash{$a}{'highest'} <=> $InAAProtein_hash{$b}{'highest'};
}
#######################
#by_highest_AAConPeptide Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub by_highest_AAConPeptide {
    #print "a $a b $b\n";
    #print "a $InAAProtein_hash{$a}{'highest'} b is $InAAProtein_hash{$b}{'highest'}\n";   
    return $InAAConPeptide_hash{$a}{'highest'} <=> $InAAConPeptide_hash{$b}{'highest'};
}
#######################
#by_highest_AAPeptide Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#
#
#######################
sub by_highest_AAPeptide {
    #print "a $a b $b\n";
    #print "a $InAAProtein_hash{$a}{'highest'} b is $InAAProtein_hash{$b}{'highest'}\n";
    return $InAAPeptide_hash{$a}{'highest'} <=> $InAAPeptide_hash{$b}{'highest'};
}
#######################
#AMTwindow Subroutine
#Richard Jacob
#Copyright (c) 2004 KBiosystems
#
#Description
#
#
#######################
sub AMTwindow {
	#GUI variables
	my (
		$FormAMT,
		$lbFilename,
		$lbTolerance,
		$lbRTTolerance,
		$btFileName,
		$edFileName,
		$edRTTolerance,
		$rbTolDa,
		$rbTolppm,
		$rbTolmmu,
		$rbFileTypeAA,
		$rbFileTypeSum,
		$AM,
		$AMT,
		$btCalAMT,
		$btCancel,
		$lboxTolerance,
		$sbAMTTolerance,
		
	);
	#variables
	my (
		$filename,
		$tolerance_type,
		@toleranceValueAMT,
		$filetype,
		$AMT_switch,
		$RTtolerance_value,
	);
	
	
	#Set variables
	$filename = "$currentPath"; 
	$tolerance_type =  "ppm";
	(@toleranceValueAMT) = &build_toleranceValue_list;
	#filetype 0 = Single AA file
	#filetype 1 = All peptide file
	$filetype = 0;
	#AMT_switch 0 = Single AA file
	#AMT_switch 1 = All peptide file
	$AMT_switch = 0;
	$RTtolerance_value = 2.5;
	
	$FormAMT = MainWindow->new;
	$FormAMT->title("Calculate AM and AMT tags");
	$FormAMT->geometry("321x237+8+8");
	$lbFilename = $FormAMT->Label(
								  -text => "File:", 
								  -background => "#A2C8A9", 
								  -cursor => "", 
								  -font => "Tahoma 8 normal"
								  );
	
	
	$lbFilename->place(-width => 31, 
					   -height => 13, 
					   -x => 16, 
					   -y => 16);
	
	$btFileName = $FormAMT->Button(
								   -text => "..", 
								   -cursor => "", 
								   -font => "Tahoma 8 normal",
								   -command => sub {($filename) = &selectAMTfile}
								   );
	
	
	$btFileName->place(-width => 19, 
					   -height => 25, 
					   -x => 176, 
					   -y => 8);
	
	$edFileName = $FormAMT->Entry(
								  -cursor => "", 
								  -font => "Tahoma 8 normal",
								  -textvariable => \$filename,
								  );
	
	
	$edFileName->place(-width => 121, 
					   -height => 21, 
					   -x => 44, 
					   -y => 8);
	
	$lbTolerance = $FormAMT->Label(
                                -text => "Tolerance", 
                                -background => "#A2C8A9", 
                                -cursor => "", 
                                -font => "Tahoma 8 normal"
                                );


	$lbTolerance->place(-width => 55, 
						 -height => 13, 
						 -x => 16, 
						 -y => 60);
	
	$lbRTTolerance = $FormAMT->Label(
                                -text => "Minutes", 
                                -background => "#A2C8A9", 
                                -cursor => "", 
                                -font => "Tahoma 8 normal"
                                );


	$lbRTTolerance->place(-width => 70, 
						 -height => 13, 
						 -x => 200, 
						 -y => 60);
	
	$edRTTolerance = $FormAMT->Entry(
								  -cursor => "", 
								  -font => "Tahoma 8 normal",
								  -textvariable => \$RTtolerance_value,
								  );
	
	
	$edRTTolerance->place(-width => 40, 
					   -height => 21, 
					   -x => 260, 
					   -y => 55);
	
	$rbTolDa = $FormAMT->Radiobutton(
									-text => "ppm",
									-value => "ppm",
									-variable => \$tolerance_type,
										 );
	
	
	$rbTolDa->place(-width => 41, 
						-height => 17, 
						-x => 83, 
						-y => 56);
	
	$rbTolppm = $FormAMT->Radiobutton(
									-text => "Da",
									-value => "Da",
									-variable => \$tolerance_type,
									);
	
	
	$rbTolppm->place(-width => 41, 
				   -height => 17, 
				   -x => 80, 
				   -y => 72);
	
	$rbTolmmu = $FormAMT->Radiobutton(
									-text => "mmu",
									-value => "mmu",
									-variable => \$tolerance_type
									);
	
	
	$rbTolmmu->place(-width => 41, 
				   -height => 17, 
				   -x => 84, 
				   -y => 88);
	
	$sbAMTTolerance = $FormAMT->Scrollbar();
	$lboxTolerance = $FormAMT->Listbox(
									 -cursor => "", 
									 -font => "Tahoma 8 normal",
									 -selectmode => "extended",
									 -exportselection =>0,
									 -yscrollcommand=>['set' => $sbAMTTolerance]
									 );
	
	
	$lboxTolerance->place(-width => 49, 
						-height => 89, 
						-x => 128, 
						-y => 56);
	
	
	$sbAMTTolerance->configure(-command => ['yview' => $lboxTolerance]);
	$sbAMTTolerance->place(-x => 177, -y => 56, -height => 89, -width => 16);
	
	$rbFileTypeAA = $FormAMT->Radiobutton(
										-text => "Single amino acid file",
										-value => "0",
										-variable => \$filetype,
										);
	
	
	$rbFileTypeAA->place(-width => 125, 
					   -height => 17, 
					   -x => 20, 
					   -y => 160);
	
	$rbFileTypeSum = $FormAMT->Radiobutton(
										-text => "Summary file",
										-value => "1",
										-variable => \$filetype,
										);
	
	
	$rbFileTypeSum->place(-width => 80, 
					   -height => 17, 
					   -x => 24, 
					   -y => 176);
	
	$AM = $FormAMT->Radiobutton(
								-text => "Accurate mass tags",
								-value => "0",
								-variable => \$AMT_switch,
									);
	
	
	$AM->place(-width => 120, 
				   -height => 17, 
				   -x => 152, 
				   -y => 160);
	
	$AMT = $FormAMT->Radiobutton(
								 -text => "Accurate mass and time tags",
								 -value => "1",
								 -variable => \$AMT_switch,
									);
	
	
	$AMT->place(-width => 161, 
				   -height => 17, 
				   -x => 152, 
				   -y => 176);
	
	$btCalAMT = $FormAMT->Button(
								 -text => "Calculate AMT's", 
								 -cursor => "", 
								 -font => "Tahoma 8 normal",
								 -command => sub {
									@toleranceValueAMT = $lboxTolerance->curselection();
									&calculateAMT ($filename, $filetype, $tolerance_type, \@toleranceValueAMT, $AMT_switch, $RTtolerance_value);
									$FormAMT->destroy}
								 );
	
	
	$btCalAMT->place(-width => 91, 
					 -height => 25, 
					 -x => 24, 
					 -y => 208);
	
	$btCancel = $FormAMT->Button(
								 -text => "Cancel", 
								 -cursor => "", 
								 -font => "Tahoma 8 normal",
								 -command => sub {$FormAMT->destroy}
								 );
	
	
	$btCancel->place(-width => 75, 
					 -height => 25, 
					 -x => 152, 
					 -y => 208);

	$lboxTolerance->insert('end', @toleranceValueAMT);
	$lboxTolerance->selectionSet(4);
	
	return ();
}
#######################
#calculateAMT Subroutine
#Richard Jacob
#Copyright (c) 2004 KBiosystems
#
#Description
#
#
#######################
sub calculateAMT {
	my ($filename, $filetype, $tolerance_type, $tolerance_values, $AMT_switch, $RTtolerance_value) = @_;
	my (
		$outfilename,
		$line,
		@line_split,
		@massList,
		$mass,
		$i,
		$peptideIndex,
		%massHash,
		@tempPep,
		$peptideIndex2,
		@AllMasses,
		$currentMass,
		$mass_window ,
		$NoMasses,
		$j,
		$nextMass,
		$low,
		$high,
		%proteinList,
		$AMTcount,
		$uniqueProCount,
		@proteinIndexArray,
		$allProteinCount,
		$allPeptideCount,
		@inFilenameSplit,
		$proteinIndex,
		@tolerance_values_list,
		$tolerance_value,
		$toleranceIndex,
		$linsplitIndex,
		$seqIndex,
		$RTIndex,
		$currentRT,
		$RTtolerance_type,
		$tempMass,
		$tempRT,
		@tempMassList,
		$AMTflag,
	);	
	$allProteinCount = 0;
	$allPeptideCount = 0;   
		
	#read in protein digest file in to hash
	open (INPUT, "$filename");
	while (<INPUT>){
		$line = $_;
		chomp $line;
		unless ($line =~ /^\d/){
			last;
		}
		$allProteinCount ++;
		@line_split = split (/\t/, $line);
		#if its a single AA file
		if ($filetype ==0){
			$linsplitIndex = 7;
			$peptideIndex = 10;
			$seqIndex = 3;
			$RTIndex = 2;
		}
		#if its a summaray file
		else {
			$linsplitIndex = 5;
			$peptideIndex = 7;
			$seqIndex = 2;
			$RTIndex = 1;
		}	
		if ($line_split[$linsplitIndex] >0){
			#make hash with mass as key and protein index No and peptide index No as array
			for ($i=1; $i<=$line_split[$linsplitIndex];$i++){
				@tempPep = ();
				$allPeptideCount ++;
				#protein index No
				push (@tempPep , $line_split[0]);
				#protein accession No
				push (@tempPep , $line_split[1]);
				#peptide sequence
				$peptideIndex2 = $peptideIndex + $seqIndex;
				push (@tempPep , $line_split[$peptideIndex2]);
				#retention time
				$peptideIndex2 = $peptideIndex + $RTIndex;
				push (@tempPep , $line_split[$peptideIndex2]);
				if ($massHash{$line_split[$peptideIndex]}){
					push (@{$massHash{$line_split[$peptideIndex]}},[@tempPep]);
				}
				else {
					@{$massHash{$line_split[$peptideIndex]}}=[@tempPep];
				}
				$peptideIndex = $peptideIndex +4;
			}
		}
	}
	close INPUT;

	#
	#make sorted list of hash keys
	@AllMasses  = sort keys (%massHash);
	$NoMasses = $#AllMasses; 
	#For each tolerance value
	(@tolerance_values_list) = &build_toleranceValue_list;
	foreach $toleranceIndex (@$tolerance_values){
		$tolerance_value = $tolerance_values_list[$toleranceIndex];
		#starting from lowest key
		#foreach mass
		$AMTcount = 0;
		$low = 0;
		%proteinList = ();
		#just the AM's please
		if ($AMT_switch ==0){
			print "Calculating AM's for $filename at $tolerance_value $tolerance_type\n";
			for ($i=0; $i<=$NoMasses;$i++){
				$high = 0;
				@tempPep = ();
				$currentMass = $AllMasses[$i];
				#determine mass error
				$mass_window = cal_mass_tolerance ($currentMass, $tolerance_type, $tolerance_value );
				if ($i==$NoMasses){
					#last mass put next mass out of range 
					$nextMass = $currentMass + 10000;
				}
				else {
					$j = $i +1;
					$nextMass = $AllMasses[$j];
				}
				#is higher mass equal or lower than tolerance value?
				if ($nextMass <= ($currentMass  + $mass_window)){
					#if yes high = 1
					$high = 1;
					$low = $high;
					next;
				}
				#Has the mass already been seen?
				elsif ($#{$massHash{$currentMass}}>=1){
					$low = $high;
					next;
				}
				#Was the mass in the range of a lower mass?
				elsif ($low == 1) {
					$low = $high;
					next;
				}
				else{
					#protein accession No
					push (@tempPep, $massHash{$currentMass}[0][1]);
					#peptide Mass
					push (@tempPep, $currentMass);
					#peptide sequence
					push (@tempPep, $massHash{$currentMass}[0][2]);
					#retention time
					push (@tempPep, $massHash{$currentMass}[0][3]);
					#index by protein index number
					$AMTcount ++;
					if ($proteinList{$massHash{$currentMass}[0][0]}){
						push(@{$proteinList{$massHash{$currentMass}[0][0]}}, [@tempPep]);
					}
					else{
						@{$proteinList{$massHash{$currentMass}[0][0]}} = [@tempPep];
					}
					$mass_window = cal_mass_tolerance ($nextMass, $tolerance_type, $tolerance_value );
					if ($currentMass >= ($nextMass - $mass_window)){
						#if yes high = 1
						$high = 1;
					}
					$low = $high;
				}
			}
		}
		#Full AMT's calculate via different algorithm
		else{
			print "Calculatin AMT's for $filename at $tolerance_value $tolerance_type\n";
			for ($i=0; $i<=$NoMasses;$i++){
				@tempPep = ();
				@tempMassList = ();
				$currentMass = $AllMasses[$i];
				#determine mass error
				$mass_window = cal_mass_tolerance ($currentMass, $tolerance_type, $tolerance_value );
				#go back up list to lower masses until masses are out of tolerance range
				$j = $i -1;
				while ($j >=0){					
					$nextMass = $AllMasses[$j];
					if ($nextMass > ($currentMass - $mass_window)){
						push (@tempMassList , $nextMass);
					}
					else {
						last;
					}
					$j--;
					
				};
				#go down list to higher masses until masses are out of tolerance range
				$j = $i +1;
				while ($j<=$NoMasses){
					$nextMass = $AllMasses[$j];
					if ($nextMass < ($currentMass + $mass_window)){
						push (@tempMassList , $nextMass);
					}
					else {
						last;
					}
					$j++;
					
				};
				#calculate retention time window for mass
				$currentRT = $massHash{$currentMass}[0][3];
				$RTtolerance_type = "Da";
				#foreach mass in temp list
				$AMTflag = 0;
				foreach $tempMass (@tempMassList){
					#get the RT for the Temp mass
					$tempRT = $massHash{$tempMass}[0][3];
					#if temp mass RT <= current mass RT
					if (  $tempRT >($currentRT + $RTtolerance_value)){
						#this mass is not in the window
						next;
					}
					else {
						if (  $tempRT <($currentRT - $RTtolerance_value)){
							#this mass is not in the window
							next;
						}
						$AMTflag = 1;
						last;
					}
				}
				if ($AMTflag ==0){
					#we've got ourselves an AMT
					#protein accession No
					push (@tempPep, $massHash{$currentMass}[0][1]);
					#peptide Mass
					push (@tempPep, $currentMass);
					#peptide sequence
					push (@tempPep, $massHash{$currentMass}[0][2]);
					#retention time
					push (@tempPep, $currentRT);
					#index by protein index number
					$AMTcount ++;
					if ($proteinList{$massHash{$currentMass}[0][0]}){
						push(@{$proteinList{$massHash{$currentMass}[0][0]}}, [@tempPep]);
					}
					else{
						@{$proteinList{$massHash{$currentMass}[0][0]}} = [@tempPep];
					}
				}
			}
		}
		
		#print protein index no, gi peptide mass and sequence to file
		@proteinIndexArray = keys (%proteinList);
		@proteinIndexArray = sort {$a<=>$b} @proteinIndexArray;
		$uniqueProCount = $#proteinIndexArray;
		$uniqueProCount ++;
		#generate output filename
		@inFilenameSplit = split (/\./, $filename);
		
		$outfilename = $inFilenameSplit[0] . "_" . $tolerance_type ."_" . $tolerance_value;
		if ($AMT_switch ==0){
			$outfilename = $outfilename . "_AM.txt"
		}
		else {
			$outfilename = $outfilename . "_AMT.txt"
		}
		open (OUTPUT, ">$outfilename");
		print OUTPUT "Index No\tAccession No\tPeptide Mass\tRentention Time\tPeptide Sequence\n";
		foreach $proteinIndex (@proteinIndexArray) {
			#print OUTPUT "$proteinIndex\t$proteinList{$proteinIndex}[0][0]\n";
			for ($i = 0; $i <= $#{$proteinList{$proteinIndex}}; $i++){
				print OUTPUT "$proteinIndex\t$proteinList{$proteinIndex}[0][0]\t$proteinList{$proteinIndex}[$i][1]\t$proteinList{$proteinIndex}[$i][3]\t$proteinList{$proteinIndex}[$i][2]\n";
			}
		}
		#print No unique proteins, percent unique proteins
		#print No AMT's, percent AMT's
		print OUTPUT "\nNumber of Proteins:\t$allProteinCount\n";
		print OUTPUT "Number of proteins identified by an AMT:\t$uniqueProCount\n";
		print OUTPUT "Number of peptides:\t$allPeptideCount\n";
		print OUTPUT "Number of AMT's:\t$AMTcount\n";
		close OUTPUT;
	}
	return ()
}

#######################
#about Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Opens "About" window with www link to KBiosystems
#
#######################
sub about {
	my $FormAbout = MainWindow->new;
	$FormAbout->title("ProteomeXplorer - About");
	$FormAbout->geometry("450x350+8+8");
	my $Label1 = $FormAbout->Label(
							-text => "ProteomeXplorer\n Version 1.0", 
							-background => "#D4D0C8", 
							-cursor => "", 
							-font => "Tahoma 14 bold"
							);
	$Label1->place(-width => 232, 
				   -height => 56, 
				   -x => 170, 
				   -y => 50);
	
	my $logokb = $FormAbout->Photo(-file=> "KBiosystemsLogo.gif");
	my $btLogokb = $FormAbout->Button(
                          -image => $logokb,
                          -command => sub { gologo("kb");},
                          -cursor => ""
                          );

	$btLogokb->place(-width => 59, 
                -height => 100, 
                -x => 4, 
                -y => 10);
	
	my $logoLudwig = $FormAbout->Photo(-file=> "LICRlogo.gif");
	my $btLogoLudwig = $FormAbout->Button(
                          -image => $logoLudwig,
                          -command => sub{gologo("Ludwig"),},
                          -cursor => ""
                          );

	$btLogoLudwig->place(-width => 96, 
                -height => 98, 
                -x => 4, 
                -y => 115);
	
	my $logoUCL = $FormAbout->Photo(-file=> "UCLlogo.gif");
	my $btLogoUCL = $FormAbout->Button(
                          -image => $logoUCL,
                          -command => sub{&gologo("UCL"),},
                          -cursor => ""
                          );

	$btLogoUCL->place(-width => 93, 
                -height => 111, 
                -x => 4, 
                -y => 220);

	my $Label2 = $FormAbout->Label(
							-text => "ProteomeXplorer - a program to do in silico digests\nto allow files identification and removal of contaminant\nmasses. The output is suitable for searching with Mascot or\nProteinProspector.\nBy Richard Jacob.\nCopyright (2003) KBiosysytems.\nAll rights reserved.", 
							-background => "#D4D0C8", 
							-cursor => "", 
							-font => "Tahoma 10 normal"
							);
	$Label2->place(-width => 350, 
				   -height => 200, 
				   -x => 100, 
				   -y => 115);

	my $Label3 = $FormAbout->Label(
							-text => "Email\: r.jacob\@ucl.ac.uk with questions and bug reports.", 
							-background => "#D4D0C8", 
							-cursor => "", 
							-font => "Tahoma 8 normal"
							);

	$Label3->place(-width => 300, 
				   -height => 30, 
				   -x => 100, 
				   -y => 300);
	return ();
}

#######################
#gologo Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#opens the logos  webpage
#
#######################
sub gologo {
	my ($logorequest) = @_;
	my %logohash = ("kb","http://www.kbiosystems.com/flash%20site/index.html", "Ludwig","http://www.ludwig.ucl.ac.uk/bachem_html/index.htm","UCL","http://www.biochem.ucl.ac.uk/" );
	if ($perlversion eq "MSWin32"){
		my $rc = ShellExecute("open", "$logohash{$logorequest}");
	}
	else {
		
	}
	return ();
}

#######################
#quitProgram Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Exit program
#
#######################
sub quitProgram {
		exit;
}
#######################
#help Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Link to help pdf
#
#######################
sub help {
	if ($perlversion eq "MSWin32"){
		my $rc = ShellExecute("open", "ProteomeXplorer User Documentation.pdf");
	}
	else{
		#no generic unix method as yet
	}
	return ();
}
#######################
#usage Subroutine
#Richard Jacob
#Copyright (c) 2004 KBiosystems
#
#Description
#
#
#######################
sub usage{
print STDERR << "eof";

    ProteomeXplorer Help...

    usage: $0 [-c] [-b] [-d file][-e enzyme] [-f db format]

	-h                   : this (help) message
	-b                   : bin the resulting peptide mass files
	-c                   : run the program on the command line
	-d file              : Database input file
	-e enzyme            : Enzyme name
	-f format            : Database format : NCBInr_Fasta, Swissprot_Fasta,
	                       IPI_Fasta or other


    example: $0 -c -b -d IPIhuman -e trypsin -f IPI_Fasta

eof
	exit;
}